import tkinter as tk
from tkinter import messagebox
import pyautogui
import keyboard
import threading
import time

# A flag to determine if the program is clicking
clicking = False

# The actual clicking logic. interval / 1000.0 because "time.sleep()" uses seconds as its base unit.
def start_clicking(interval):
    global clicking
    while clicking:
        pyautogui.click()
        time.sleep(interval / 1000.0)

def toggle_clicking():
    global clicking
    if clicking:
        clicking = False
        start_stop_button.config(text="Start Clicking")
    else:
        try:
            interval = int(interval_entry.get())
            if interval <= 0:
                raise ValueError
            hotkey = hotkey_entry.get()
            if not hotkey:
                raise ValueError("Hotkey cannot be empty.")
            clicking = True
            start_stop_button.config(text="Stop Clicking")
            threading.Thread(target=start_clicking, args=(interval,)).start()
        except ValueError:
            clicking = False
            messagebox.showerror("Invalid Input", "Please enter a valid integer >= 10 for the click interval and a non-empty hotkey.")

# Function to handle the Start Clicking button changing its text, and disabling it temporarily after starting clicking
def toggle_clicking_button():
    global clicking
    if not clicking:
        try:
            interval = int(interval_entry.get())
            if interval <= 0:
                raise ValueError
            clicking = True
            start_stop_button.config(text="Stop Clicking", state=tk.DISABLED)
            threading.Thread(target=start_clicking, args=(interval,)).start()
            root.after(1000, lambda: start_stop_button.config(state=tk.NORMAL))
        except ValueError:
            messagebox.showerror("Invalid Input", "Interval must be a positive integer >= 10.")
    else:
        clicking = False
        start_stop_button.config(text="Start Clicking")

# Initialize the default values
def initialize_autoclicker():
    default_hotkey = "X"
    default_interval = 1000
    interval_entry.insert(0, str(default_interval))
    hotkey_entry.insert(0, default_hotkey)
    keyboard.add_hotkey(default_hotkey, toggle_clicking)

# Start autoclicker function
def start_autoclicker():
    try:
        hotkey = hotkey_entry.get()
        interval = interval_entry.get()

        # Checks if hotkey and interval have input
        if not hotkey and not interval:
            raise ValueError("Hotkey and interval cannot be empty.")
        elif not hotkey:
            raise ValueError("Hotkey cannot be empty.")
        elif not interval:
            raise ValueError("Interval cannot be empty.")
        
        # Checks if interval is numeric
        try:
            interval = int(interval)
            if interval <= 9:
                raise ValueError("Interval must be a positive integer >= 10.")
        except ValueError:
            raise ValueError("Interval must be a positive integer >= 10.")

        keyboard.add_hotkey(hotkey, toggle_clicking)
        messagebox.showinfo("Hotkey Set", f"Auto-clicker hotkey set to: {hotkey}, interval set to {interval}.\nPress the hotkey to start/stop clicking.")
    except ValueError as e:
        messagebox.showerror("Invalid Input", str(e))

def clear_focus(event):
    # Function for clearing cursor from entry widgets
    if not isinstance(event.widget, tk.Entry):
        root.focus_set()

def show_help_message():
    messagebox.showinfo("Instructions", "The default interval is 1000 ms (1 second) with the hotkey being X. You can customize this to your liking by changing the value and clicking \"Apply\". Pressing the hotkey will toggle the auto clicker. You can also click \"Start/Stop Clicking\" to toggle clicking. Please be advised that due to limitations with Python, the interval must be >= 10.")

# Create the main window
root = tk.Tk()
root.title("Ohooh Autoclick")
root.geometry("400x250")

root.bind('<Button-1>', clear_focus)

# Title label
label = tk.Label(root, text="Ohooh Autoclick")
label.place(x=120, y=10)

# Help button
help_button = tk.Button(root, text="   Help   ", command=show_help_message)
help_button.place(x=298, y=7)

# Apply settings button
button = tk.Button(root, text="     Apply     ", command=start_autoclicker)
button.place(x=270, y=185)

# Start/stop clicking button
start_stop_button = tk.Button(root, text="Start Clicking", command=toggle_clicking_button)
start_stop_button.place(x=20, y=185)

# Click Interval label
interval_label = tk.Label(root, text="Click Interval (ms):")
interval_label.place(x=10, y=70)
# Click Interval entry
interval_entry = tk.Entry(root, width=12)
interval_entry.place(x=180, y=70)

# Hotkey label
hotkey_label = tk.Label(root, text="Hotkey:")
hotkey_label.place(x=10, y=110)
# Hotkey entry
hotkey_entry = tk.Entry(root, width=12)
hotkey_entry.place(x=180, y=110)

initialize_autoclicker()

root.mainloop()